//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef CONFIGMANAGER_HPP
#define CONFIGMANAGER_HPP
//------------------------------------------------------------------------------
#include <ticpp.h>
#include <ticpprc.h>

#include <map>
#include <vector>
#include <string>
#include <queue>
#include <set>
#include <SFML/Graphics.hpp>
#include "ClassAllocCounter.hpp"

#include <luabind/object.hpp>

#include "config.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** A class that handle arguments passed to the mode and arguments to be passed to the next mode. */
/// LUA_HEADER_COPY_OFF ///
class Arguments
{
public:
    /** Constructor */
    Arguments();

    /// LUA_EXPORT_ON(Arguments) ///
    /** Get the number of argument */
    int GetCount() const;

    /** Get the next argument (or the first one if you call this function for the first time). */
    std::string GetNext();

    /** Add an argument at the end of the list. */
    void Add(const std::string &arg);

    /** Clear all stored arguments */
    void Clear();
    /// LUA_EXPORT_OFF ///
protected:
    std::queue<std::string> m_argList;
};
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** A structure containing information about a mode. */
/// LUA_HEADER_COPY_OFF ///
struct ModeInfo
{
    /// LUA_EXPORT_ON(ModeInfo) ///
    //LUA// luaReadwrite(string, fullName);
    //LUA// luaReadwrite(string, idName);
    //LUA// luaReadwrite(string, auther);
    //LUA// luaReadwrite(string, path);
    //LUA// luaReadwrite(string, description);
    //LUA// luaReadwrite(string, version);
    //LUA// luaReadwrite(string, screenshot);
    /// LUA_EXPORT_OFF ///

    std::string fullName;
    std::string idName;
    std::string auther;
    std::string path;
    std::string description;
    std::string version;
    std::string screenshot;
};
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** A config node represent a node in the configuration file (like an XML node). You can add data to the node with a key and add child node. */
/// LUA_HEADER_COPY_OFF ///
class ConfigNode
{
public:
    // Constructor
    /** Construct ConfigMode from a NULL ticpp Element */
    ConfigNode();

    /** Construct ConfigMode from a ticpp Element */
    ConfigNode(ticpp::Element *node);

    /** Copy constructor */
    ConfigNode(const ConfigNode &src);

    /** Destructor */
    virtual ~ConfigNode();

    /// LUA_EXPORT_ON(ConfigNode) ///
    // Lua
    // Get
    /** Get the string corresponding to key in this node */
    std::string GetString(const std::string &key) const;

    /** Get the number corresponding to key in this node */
    float GetNumber(const std::string &key) const;

    /** Get the value corresponding to key in this node. Value type can be a boolean, a string, a number, nil or a Vector2D. */
    luabind::object GetValue(const std::string &key) const;

    /** Retrun true if the child key exists (it can be string / number / child node) */
    bool ChildExists(const std::string &key) const;

    /** Return a lua table containing the values of this node and its children */
    luabind::object GetLuaTable() const;

    // Set
    /** Set / create a value with key. Value type can be a boolean, a string, a number, nil or a Vector2D. */
    void SetValue(const std::string &key, luabind::object value);

    /** Set a lua table as a value. All element of the table will be saved in this node. Table elements types can be a boolean, a string, a number, nil, Vector2D or another table. Note: the node will not be cleared. */
    void SetLuaTable(luabind::object table);

    // Child node
    /** Get the ConfigNode corresponding to key, if it exists, otherwise, create a child ConfigNode named "key" in this ConfigNode and return it */
    ConfigNode AccessNode(const std::string &key);

    // Remove
    /** Remove the child named key */
    void Remove(const std::string &key);

    /** Clear the ConfigNode (delete all childs) */
    void Clear();

    /** Return true is this Node or one of its parents has been deleted */
    bool IsDeleted() const;
    /// LUA_EXPORT_OFF ///

    // No Lua
    void SetXmlNode(ticpp::Element *node);
    ticpp::Element *GetXmlNode();


    /** Set / create a number with key */
    void SetValue(const std::string &key, const std::string &value);

    /** Set / create a number with key */
    void SetValue(const std::string &key, float value);


    /** Get a number and write default value if it doesn't exist */
    float GetNumberDefault(const std::string &key, float defaultVal);

    /** Notice that this node (and all its children) are know destroyed / invalid */
    void NoticeNodeDestroyed();

    void ThrowIfNotOk() const;
    bool IsPtrOk() const;

    #if DEBUG == 1
    static int GetProtectedElementsCount();
    static void PrintLeftProtectedElements();
    #endif

protected:
    void InvalidatePtr();

    static luabind::object GetValueFromElem(ticpp::Element *elem);
    static void DestroyAllAttributes(ticpp::Element *elem);

    ticpp::Element *m_node;

    // Mem protect
    bool m_isPtrOk;

    typedef std::string UniqueElementId;

    typedef std::map<UniqueElementId, std::set<ConfigNode *> > ProtectionMap;
    static ProtectionMap m_protectionMap;

    static UniqueElementId GetUniqueElementId(const ticpp::Node *elem);
    void NoticeDeletionOfOneElement(ticpp::Element* elem);
    void NoticeDeletionOfChildren(ticpp::Element* parent);
    void NoticeDeletionOfThisAndChildren(ticpp::Element* parent);

private:
    ConfigNode& operator=(const ConfigNode &obj);
};
//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** The config manager is designed to contain the list of the mode and their information. It also contain the configuration node shared between all mode. Textdomain sould be bottle2d !*/
/// LUA_HEADER_COPY_OFF ///
class ConfigManager
{
public:
    ConfigManager();
    virtual ~ConfigManager();

    /// LUA_EXPORT_ON(ConfigManager) ///
    // Lua
    /** Get information about mode idName */
    ModeInfo GetModeInfo(const std::string &idName);

    /** Get information about mode number id */
    ModeInfo GetModeInfo(int id);

    /** Get the current mode id */
    std::string GetCurrentModeId() const;

    /** Get the root ConfigNode for the current mode */
    ConfigNode GetRootConfigNode();

    /** Return the number of mode */
    int GetModeCount() const;

    /** Get the global ConfigNode (Configuration shared between every mode) */
    ConfigNode GetGlobalConfigNode();

    /** Init the locales. Path should contain the language dir (like en, fr...). Usualy, Path is "<ModePath>/local/" */
    void InitLocales(const std::string &path);
    /// LUA_EXPORT_OFF ///

    // No Lua
    /** Create a new RenderWindow, wich should be deleted by the programm */
    sf::RenderWindow *CreateRenderWindow();

    /** Select a mode to use. It will load the mode configuration. */
    void LoadMode(const std::string &idName);

    /** Load mode list and global config */
    void LoadAll();

    /** Get the path of the common data dir (data/common) */
    std::string GetCommonDataPath();

    /** Get the path of the mode dir (data/modes/[modeName]) */
    std::string GetModePath();

    /** Get the argument object */
    Arguments &GetArguments();

protected:
    static std::vector<std::string> GetDirsInPath(const std::string &path);
    std::vector<std::string> GetConfigDirs();

    static void LoadXmlConfig(const std::string &filename, ticpp::Document &doc, ConfigNode &rootNode);
    static void SaveXmlConfig(ticpp::Document &doc, ConfigNode &node);

    /** Detect avaible modes */
    void LoadModeList();

    /** Load useful paths. Should be called first */
    void LoadStandardPaths();

protected:
    // Mode
    ModeInfo *m_currentMode;

    // Mode config
    ticpp::Document m_modeConfigDoc;
    ConfigNode m_modeConfig;

    // Global config ; can't be accessed from Lua
    ticpp::Document m_globalConfigDoc;
    ConfigNode m_globalConfig;

    // Mode list
    std::map<std::string, ModeInfo> m_modList;

    // Std paths
    std::string m_dataPath, m_userConfigPath, m_dataCommonPath;

    // Argument list
    Arguments m_arguments;

    // Icon image
    sf::Image m_iconImg;

private:
    // Non copiable
    ConfigManager(ConfigManager &);
    ConfigManager &operator=(ConfigManager &);

};
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
#endif // CONFIGMANAGER_HPP
