//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef SHAPE_HPP
#define SHAPE_HPP

#include <SFML/Graphics.hpp>

#include "Drawable.hpp"
//------------------------------------------------------------------------------
namespace En
{
/// LUA_HEADER_COPY_ON ///
/** A Shape is a basic shape with a color and an outline color. You can add and remove as much point as you want to make every shape you want.\\n
  * Use the function Group::CreateShape to create an empty shape.\\n
  * The following functions can be used to create basic shapes:
  * - Group::CreateShapeLine
  * - Group::CreateShapeRectangle
  * - Group::CreateShapeCircle
  */
/// LUA_HEADER_COPY_OFF ///

class Shape : public DrawableSF, public sf::Shape
{
public:
    Shape(Group *parent, const std::string &name);
    static Shape *Create(Group *parent, const std::string &name) {return new Shape(parent, name);}

    /// LUA_EXPORT_ON(Shape, Drawable, ProtectedPtr<Object>) ///

    // -------------- From sf::Drawable --------------
    /** Acces object's color. */
    //LUA// luaProperty(Color, color, GetColor, SetColor);
    /** Set object's color. */
    //LUA// void SetColor(const Color& Col);
    /** Get object's color. */
    Color GetColor() const {return sf::Drawable::GetColor();}
    /** Set object's scale */
    void SetScale(const Vector2D& scale) {sf::Shape::SetScale(scale);}


    /** Get object's scale */
    Vector2D GetScale() const {return sf::Drawable::GetScale();}
    /** Set object's scale (x) */
    //LUA// void SetScaleX(float FactorX);
    /** Set object's scale (y) */
    //LUA// void SetScaleY(float FactorY);
    /** Set object's center */
    void SetCenter(const Vector2D& center) {sf::Shape::SetCenter(center);}
    /** Get object's center */
    Vector2D GetCenter() const {return sf::Drawable::GetCenter();}

    // From sf::Shape
    // /** Add a point to the shape */
    // //LUA// void AddPoint(float X, float Y, const Color& Col, const Color& OutlineCol);
    /** Add a point to the shape. */
    void AddPoint(const Vector2D& Position, const Color& Col, const Color& OutlineCol) {sf::Shape::AddPoint(Position, Col, OutlineCol);}
    /** Get the total number of point in the shape. */
    //LUA// unsigned int GetNbPoints() const;
    /** Enable / disable filling the shape with a color. */
    //LUA// void EnableFill(bool Enable);
    /** Enable / disable outline. */
    //LUA// void EnableOutline(bool Enable);
    /** Change the position of a point. Index starts at 1. */
    void SetPointPosition(int index, const Vector2D& Position);
    /** Change the color of a point. Index starts at 1. */
    void SetPointColor(int index, const Color& Col);
    /** Change the outline color of a point. Index starts at 1. */
    void SetPointOutlineColor(int index, const Color& OutlineCol);
    /** Change the outline width */
    //LUA// void SetOutlineWidth(float Width);
    /** Get a point position. Index starts at 1. */
    Vector2D GetPointPosition(int index) const;
    /** Get a point color. Index starts at 1. */
    Color GetPointColor(int index) const;
    /** Get a point outline color. Index starts at 1. */
    Color GetPointOutlineColor(int index) const;
    /** Get the width of the outline */
    //LUA// float GetOutlineWidth() const;

    /** Initialise the shape as a line.\\n
      * \\param P1 Position of the first point
      * \\param P2 Position second point
      * \\param Thickness Line thickness
      * \\param Col Color used to draw the line
      * \\param Outline Outline width (0 by default)
      * \\param OutlineCol Color used to draw the outline
      */

    void MakeLine(const Vector2D& P1, const Vector2D& P2, float Thickness, const Color& Col, float Outline, const Color& OutlineCol);
    /** Initialise the shape as a rectangle.\\n
      * \\param P1 Position of the first point
      * \\param P2 Position second point
      * \\param Col Color used to fill the rectangle
      * \\param Outline Outline width
      * \\param OutlineCol Color used to draw the outline
      */

    void MakeRectangle(const Vector2D& P1, const Vector2D& P2, const Color& Col, float Outline, const Color& OutlineCol);
    /** Initialise the shape as a circle / disk.\\n
      * \\param Center Position of the center
      * \\param Radius Radius
      * \\param Col Color used to fill the circle
      * \\param Outline Outline width
      * \\param OutlineCol Color used to draw the outline
      */
    void MakeCircle(const Vector2D& Center, float Radius, const Color& Col, float Outline, const Color& OutlineCol);

    /** Change the size of a rectangle shape (for compatibility with Sprite). P1 position is kept. */
    void SetSize(Vector2D &size);


    /// LUA_EXPORT_OFF ///
    // No Lua
    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual Shape *GetRawCloneToGroup(Group *newParent) const {Shape *obj = new Shape(*this); obj->RegisterClone(newParent); return obj;}

    // Acces to position & rotation
    Vector2D GetPosition() const {return DrawableSF::GetPosition();}
    virtual void SetPosition(const Vector2D &pos) {DrawableSF::SetPosition(pos);}
    float GetRotation() const {return DrawableSF::GetRotation();}
    virtual void SetRotation(float angle) {DrawableSF::SetRotation(angle);}
    void Rotate(float angle) {DrawableSF::Rotate(angle);}
    void Move(const Vector2D &mvt) {DrawableSF::Move(mvt);}

    // Render
    virtual void Render(sf::RenderTarget &App) {RenderSf(App, this);}

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_DRAWABLE_SHAPE)

protected:
};
//------------------------------------------------------------------------------
} // namespace En

#endif // SHAPE_HPP
